<?php

namespace Migrator\Happytenant\Http\Controllers;

use Exception;
use Migrator\Happytenant\Actions\UnitWorkflow;
use Migrator\Happytenant\Http\Requests\UnitWorkflowRequest;
use Migrator\Happytenant\Http\Resources\UnitResource;
use Migrator\Happytenant\Models\Unit;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class UnitWorkflowController extends BaseController
{
    /**
     * Action controller function for unit.
     *
     * @param Model $unit
     * @param action next action for the unit.
     *
     * @return Response
     */
    public function __invoke(UnitWorkflowRequest $request, Unit $unit, $trasition)
    {
        try {
            $request = $request->all();
            $unit = UnitWorkflow::run($trasition, $unit, $request);
            $data = new UnitResource($unit);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('happytenant::unit.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('happytenant/unit/' . $unit->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('happytenant/unit/' . $unit->getRouteKey()))
                ->redirect();
        }
    }
}
