<?php
namespace Migrator\Happytenant\Notifications;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Notifications\Messages\MailMessage;
use Illuminate\Notifications\Notification;
use Migrator\Happytenant\Events\UnitAction as UnitActionEvent;
use Migrator\Happytenant\Models\Unit;

class UnitAction extends Notification implements ShouldQueue
{
    use Queueable;

    protected $unit;

    protected $action;

    /**
     * Create a new notification instance.
     *
     * @param UnitActionEvent $unit
     * @param string $action
     */
    public function __construct(UnitActionEvent $unit, string $action)
    {
        $this->action = $action;
        $this->unit = $unit;
    }

    /**
     * Get the mail representation of the notification.
     *
     * @param mixed $notifiable
     * @return \Illuminate\Notifications\Messages\MailMessage
     */
    public function toMail($notifiable)
    {
        return (new MailMessage)
            ->subject('Unit Approved - ' . config('app.name'))
            ->view('happytenant::unit.email.action', ['data' => $this->unit]);
    }

    /**
     * Get the notification's delivery channels.
     *
     * @param object $notifiable
     * @return array<int, string>
     */
    public function via(object $notifiable): array
    {
        return ['mail'];
    }
}
