<?php

namespace Migrator\Happytenant\Policies;

use Litepie\User\Interfaces\UserPolicyInterface;
use Migrator\Happytenant\Models\Unit;

class UnitPolicy
{

    use UnitAction;
    use UnitReport;
    use UnitWorkflow;

    /**
     * Determine if the given user can view the unit.
     *
     * @param UserPolicyInterface $authUser
     * @param Unit $unit
     *
     * @return bool
     */
    public function view(UserPolicyInterface $authUser, Unit $unit)
    {
        if ($authUser->canDo('happytenant.unit.view') && $authUser->isAdmin()) {
            return true;
        }

        return $unit->user_id == user_id() && $unit->user_type == user_type();
    }

    /**
     * Determine if the given user can create a unit.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function create(UserPolicyInterface $authUser)
    {
        return  $authUser->canDo('happytenant.unit.create');
    }

    /**
     * Determine if the given user can update the given unit.
     *
     * @param UserPolicyInterface $authUser
     * @param Unit $unit
     *
     * @return bool
     */
    public function update(UserPolicyInterface $authUser, Unit $unit)
    {
        if ($authUser->canDo('happytenant.unit.edit') && $authUser->isAdmin()) {
            return true;
        }

        return $unit->user_id == user_id() && $unit->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given unit.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function destroy(UserPolicyInterface $authUser, Unit $unit)
    {
        return $unit->user_id == user_id() && $unit->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given unit.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function verify(UserPolicyInterface $authUser, Unit $unit)
    {
        if ($authUser->canDo('happytenant.unit.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given unit.
     *
     * @param UserPolicyInterface $authUser
     *
     * @return bool
     */
    public function approve(UserPolicyInterface $authUser, Unit $unit)
    {
        if ($authUser->canDo('happytenant.unit.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $authUser    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($authUser, $ability)
    {
        if ($authUser->isSuperuser()) {
            return true;
        }
    }
}
