<?php

namespace Migrator\Happytenant\Providers;

use Illuminate\Support\ServiceProvider;
use Migrator\Happytenant\Happytenants;

class HappytenantServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = true;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'happytenant');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'happytenant');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerFacade();

        $this->app->register(\Migrator\Happytenant\Providers\AuthServiceProvider::class);
        $this->app->register(\Migrator\Happytenant\Providers\RouteServiceProvider::class);
        $this->app->register(\Migrator\Happytenant\Providers\ActionServiceProvider::class);
        $this->app->register(\Migrator\Happytenant\Providers\WorkflowServiceProvider::class);
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->bind('migrator.happytenant', function($app)
        {
            return $this->app->make(Happytenant::class);
        });
    }

    /**
     * Merges user's and happytenant's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'migrator.happytenant'
        );
        
        
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/unit.php', 'migrator.happytenant.unit'
        );
    }


    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['migrator.happytenant'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/' => config_path('migrator/happytenant')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/happytenant')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/happytenant')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
